@@ -1,4 +1,7 @@
-/* Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
+/* SPDX-License-Identifier: GPL-2.0
+ *
+ * Copyright (C) 2015-2018 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved.
+ */
 
 #include "ratelimiter.h"
 #include <linux/siphash.h>
@@ -9,6 +12,7 @@
 static struct kmem_cache *entry_cache;
 static hsiphash_key_t key;
 static spinlock_t table_lock = __SPIN_LOCK_UNLOCKED("ratelimiter_table_lock");
+static DEFINE_MUTEX(init_lock);
 static atomic64_t refcnt = ATOMIC64_INIT(0);
 static atomic_t total_entries = ATOMIC_INIT(0);
 static unsigned int max_entries, table_size;
@@ -57,12 +61,12 @@ static void gc_entries(struct work_struct *work)
 
 	for (i = 0; i < table_size; ++i) {
 		spin_lock(&table_lock);
-		hlist_for_each_entry_safe (entry, temp, &table_v4[i], hash) {
+		hlist_for_each_entry_safe(entry, temp, &table_v4[i], hash) {
 			if (unlikely(!work) || now - entry->last_time_ns > NSEC_PER_SEC)
 				entry_uninit(entry);
 		}
 #if IS_ENABLED(CONFIG_IPV6)
-		hlist_for_each_entry_safe (entry, temp, &table_v6[i], hash) {
+		hlist_for_each_entry_safe(entry, temp, &table_v6[i], hash) {
 			if (unlikely(!work) || now - entry->last_time_ns > NSEC_PER_SEC)
 				entry_uninit(entry);
 		}
@@ -94,13 +98,14 @@ bool ratelimiter_allow(struct sk_buff *skb, struct net *net)
 	else
 		return false;
 	rcu_read_lock();
-	hlist_for_each_entry_rcu (entry, bucket, hash) {
+	hlist_for_each_entry_rcu(entry, bucket, hash) {
 		if (entry->net == net && entry->ip == data.ip) {
 			u64 now, tokens;
 			bool ret;
 			/* Inspired by nft_limit.c, but this is actually a slightly different
 			 * algorithm. Namely, we incorporate the burst as part of the maximum
-			 * tokens, rather than as part of the rate. */
+			 * tokens, rather than as part of the rate.
+			 */
 			spin_lock(&entry->lock);
 			now = ktime_get_ns();
 			tokens = min_t(u64, TOKEN_MAX, entry->tokens + now - entry->last_time_ns);
@@ -142,6 +147,7 @@ int ratelimiter_init(void)
 	if (atomic64_inc_return(&refcnt) != 1)
 		return 0;
 
+	mutex_lock(&init_lock);
 	entry_cache = KMEM_CACHE(ratelimiter_entry, 0);
 	if (!entry_cache)
 		goto err;
@@ -149,7 +155,8 @@ int ratelimiter_init(void)
 	/* xt_hashlimit.c uses a slightly different algorithm for ratelimiting,
 	 * but what it shares in common is that it uses a massive hashtable. So,
 	 * we borrow their wisdom about good table sizes on different systems
-	 * dependent on RAM. This calculation here comes from there. */
+	 * dependent on RAM. This calculation here comes from there.
+	 */
 	table_size = (totalram_pages > (1U << 30) / PAGE_SIZE) ? 8192 : max_t(unsigned long, 16, roundup_pow_of_two((totalram_pages << PAGE_SHIFT) / (1U << 14) / sizeof(struct hlist_head)));
 	max_entries = table_size * 8;
 
@@ -167,20 +174,23 @@ int ratelimiter_init(void)
 
 	queue_delayed_work(system_power_efficient_wq, &gc_work, HZ);
 	get_random_bytes(&key, sizeof(key));
+	mutex_unlock(&init_lock);
 	return 0;
 
 err_kmemcache:
 	kmem_cache_destroy(entry_cache);
 err:
 	atomic64_dec(&refcnt);
+	mutex_unlock(&init_lock);
 	return -ENOMEM;
 }
 
 void ratelimiter_uninit(void)
 {
-	if (atomic64_dec_return(&refcnt))
+	if (atomic64_dec_if_positive(&refcnt))
 		return;
 
+	mutex_lock(&init_lock);
 	cancel_delayed_work_sync(&gc_work);
 	gc_entries(NULL);
 	rcu_barrier();
@@ -189,6 +199,7 @@ void ratelimiter_uninit(void)
 	kvfree(table_v6);
 #endif
 	kmem_cache_destroy(entry_cache);
+	mutex_unlock(&init_lock);
 }
 
 #include "selftest/ratelimiter.h"
